package com.ynet.life.action;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/* loaded from: com_ynet_life_action.jar:com/ynet/life/action/IdWorker.class */
public class IdWorker {
    private long workerId;
    private long datacenterId;
    private long sequence = 0;
    private long twepoch = 1288834974657L;
    private long workerIdBits = 5;
    private long datacenterIdBits = 5;
    private long maxWorkerId = (-1) ^ ((-1) << ((int) this.workerIdBits));
    private long maxDatacenterId = (-1) ^ ((-1) << ((int) this.datacenterIdBits));
    private long sequenceBits = 12;
    private long workerIdShift = this.sequenceBits;
    private long datacenterIdShift = this.sequenceBits + this.workerIdBits;
    private long timestampLeftShift = (this.sequenceBits + this.workerIdBits) + this.datacenterIdBits;
    private long sequenceMask = (-1) ^ ((-1) << ((int) this.sequenceBits));
    private long lastTimestamp = -1;
    protected static final Logger LOG = LoggerFactory.getLogger(IdWorker.class);
    private static IdWorker instance = null;

    public static IdWorker getInstance() {
        if (instance == null) {
            instance = new IdWorker(1L, 1L);
        }
        return instance;
    }

    public IdWorker(long workerId, long datacenterId) {
        if (workerId > this.maxWorkerId || workerId < 0) {
            throw new IllegalArgumentException(String.format("worker Id can't be greater than %d or less than 0", Long.valueOf(this.maxWorkerId)));
        }
        if (datacenterId > this.maxDatacenterId || datacenterId < 0) {
            throw new IllegalArgumentException(String.format("datacenter Id can't be greater than %d or less than 0", Long.valueOf(this.maxDatacenterId)));
        }
        this.workerId = workerId;
        this.datacenterId = datacenterId;
        LOG.info(String.format("worker starting. timestamp left shift %d, datacenter id bits %d, worker id bits %d, sequence bits %d, workerid %d", Long.valueOf(this.timestampLeftShift), Long.valueOf(this.datacenterIdBits), Long.valueOf(this.workerIdBits), Long.valueOf(this.sequenceBits), Long.valueOf(workerId)));
    }

    public synchronized long nextId() {
        long timeGen = timeGen();
        long timestamp = timeGen;
        if (timeGen < this.lastTimestamp) {
            LOG.error(String.format("clock is moving backwards.  Rejecting requests until %d.", Long.valueOf(this.lastTimestamp)));
            throw new RuntimeException(String.format("Clock moved backwards.  Refusing to generate id for %d milliseconds", Long.valueOf(this.lastTimestamp - timestamp)));
        }
        if (this.lastTimestamp == timestamp) {
            this.sequence = (this.sequence + 1) & this.sequenceMask;
            if (this.sequence == 0) {
                timestamp = tilNextMillis(this.lastTimestamp);
            }
        } else {
            this.sequence = 0L;
        }
        this.lastTimestamp = timestamp;
        return ((timestamp - this.twepoch) << ((int) this.timestampLeftShift)) | (this.datacenterId << ((int) this.datacenterIdShift)) | (this.workerId << ((int) this.workerIdShift)) | this.sequence;
    }

    protected long tilNextMillis(long lastTimestamp) {
        long timestamp;
        do {
            timestamp = timeGen();
        } while (timestamp <= lastTimestamp);
        return timestamp;
    }

    protected long timeGen() {
        return System.currentTimeMillis();
    }

    public static void main(String[] args) {
        System.out.println(getInstance().nextId());
    }
}
